import os
import asyncio
from typing import Dict
from pydantic import BaseModel
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
import json

# Load API key from environment variable
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
if not OPENAI_API_KEY:
    raise ValueError("Missing OPENAI_API_KEY. Set it as an environment variable.")

# Define the expected response format as a Pydantic model
class AgentResponse(BaseModel):
    my_share: int
    other_share: int
    motivation: str

# The dictator game simulation class
class Dictator:
    def __init__(self, amount: int, model: str, temperature: float, strategy=False, max_retries: int = 3):
        self.amount = amount
        self.model = model
        self.temperature = temperature
        self.strategy = strategy
        self.max_retries = max_retries  # Maximum retry attempts in case of hallucinations

        if not strategy:
            is_openai_model = model.startswith("gpt")
            base_url = "https://api.openai.com/v1" if is_openai_model else "http://localhost:11434/v1"

            model_info = {
                "temperature": self.temperature,
                "function_calling": True,
                "parallel_tool_calls": True,
                "family": "unknown",
                "json_output": True,
                "vision": False
            }

            self.model_client = OpenAIChatCompletionClient(
                model=self.model,
                base_url=base_url,
                api_key=OPENAI_API_KEY,
                model_info=model_info,
                response_format=AgentResponse
            )

    async def run(self) -> Dict:
        """Runs the model if strategy is False, otherwise uses a classical method."""
        if self.strategy:
            return self.apply_strategy()

        instruction = f"""
        Hi, let’s play a game. You are paired with another player.
        Your role is to decide how to divide ${self.amount}, and the other player simply receives your choice.
        How would you like to divide the money?
        Please provide one specific choice, stating how much you keep and how much you give to the other player.
        Return the response in JSON format with `my_share`, `other_share`, and `motivation`.
        """

        for attempt in range(self.max_retries):
            agent = AssistantAgent(
                name="Dictator",
                model_client=self.model_client,
                system_message="You are a helpful assistant."
            )

            response = await agent.on_messages(
                [TextMessage(content=instruction, source="user")],
                cancellation_token=CancellationToken(),
            )

            try:
                response_data = response.chat_message.content
                agent_response = AgentResponse.model_validate_json(response_data)  # Parse JSON
                my_share, other_share = agent_response.my_share, agent_response.other_share

                # Validate values
                if 0 <= my_share <= self.amount and 0 <= other_share <= self.amount and my_share + other_share <= self.amount:
                    return agent_response.model_dump()
                else:
                    print(f"Invalid response detected (Attempt {attempt+1}): {response_data}")
            except Exception as e:
                print(f"Error parsing response (Attempt {attempt+1}): {e}")

        raise ValueError("Model failed to provide a valid response after multiple attempts.")

    def apply_strategy(self) -> Dict:
        """Generates a response."""
        if (self.model == "gpt-4.5-preview-2025-02-27") :
            my_share = int(0.7 * self.amount)  # Example rule: keep 70%
            other_share = self.amount - my_share
            motivation = f"Using strategy from {self.model}, I chose to keep {my_share} and give {other_share}."
            return {"my_share": my_share, "other_share": other_share, "motivation": motivation}
        if (self.model == "llama3") :
            my_share = self.amount / 2
            other_share = self.amount - my_share
            motivation = "I'm being fair and generous!"
            agent_response = AgentResponse(my_share=my_share, other_share=other_share, motivation=motivation)
            return agent_response.model_dump()
        if (self.model == "mistral-small") :
            my_share = self.amount // 2
            other_share = self.amount - my_share
            motivation = "The decision is to divide the money equally."
            agent_response = AgentResponse(
                my_share=my_share,
                other_share=other_share,
                motivation=motivation
            )
            return agent_response
        if (self.model == "deepseek-r1") :
            half_amount = self.amount // 2
            return {
                "my_share": half_amount,
                "other_share": half_amount,
                "motivation": "Split equally between both players."
            }

# Run the async function and return the response
if __name__ == "__main__":
    game_agent = Dictator(amount=100, model="laama3", temperature=0.7, strategy=True)  # Toggle strategy here
    response_json = asyncio.run(game_agent.run())
    print(response_json)
