import os
import asyncio
import csv
import random
from typing import Dict, Literal, List, Callable
from pydantic import BaseModel, ValidationError
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
import json

# Load API key from environment variable
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
if not OPENAI_API_KEY:
    raise ValueError("Missing OPENAI_API_KEY. Set it as an environment variable.")

CSV_FILE_PATH = "../../data/rps/rps.csv"

# Define the expected response format as a Pydantic model
class AgentResponse(BaseModel):
    prediction: Literal["Rock", "Paper", "Scissor"]
    reasoning: str

# Define Guess simulation class
class Guess:
    def __init__(self, model: str, temperature: float, game_id: int, opponent_strategy_fn: Callable[[List[Dict]], str], max_retries: int = 3):
        self.model = model
        self.temperature = temperature
        self.game_id = game_id
        self.max_retries = max_retries
        self.history: List[Dict] = []
        self.player_score_game = 0
        self.opponent_strategy_fn = opponent_strategy_fn

        if model == "random":
            self.model_client = None
        else:
            is_openai_model = model.startswith("gpt")
            base_url = "https://api.openai.com/v1" if is_openai_model else "http://localhost:11434/v1"
            model_info = {
                "temperature": self.temperature,
                "function_calling": True,
                "parallel_tool_calls": True,
                "family": "unknown",
                "json_output": True,
                "vision": False
            }
            self.model_client = OpenAIChatCompletionClient(
                model=self.model,
                base_url=base_url,
                api_key=OPENAI_API_KEY,
                model_info=model_info,
                response_format=AgentResponse
            )

    async def play_round(self, round_id: int) -> Dict:
        """Plays a single round of Guess The Next Move."""
        opponent_move = self.opponent_strategy_fn(self.history)

        if self.model == "random":
            move = random.choice(["Rock", "Paper", "Scissor"])
            reasoning = ""
        else:
            history_summary = self.get_history_summary()
            instruction = f"""
            We are playing a simplified version of Rock-Paper-Scissors (RPS) designed to evaluate your ability to predict the opponent's next move.

            ### **Game Rules:**
            1. The opponent follows a hidden strategy (which may involve a repeating pattern or adaptive behavior).
            2. Your task is to predict the opponent’s next move: Rock, Paper, or Scissors.
            3. If your prediction is correct, you earn **1 point**; otherwise, you earn **0 points**.
            4. The game continues for multiple rounds, and your accuracy is evaluated at each round.

            ### **Game History So Far:**
            {history_summary}

            ### **Your Task:**
            Based on the game history, predict the opponent's next move.  
            Return your response in JSON format with two keys:  
            - `"prediction"`: Your predicted move (`"Rock"`, `"Paper"`, or `"Scissor"`).  
            - `"reasoning"`: A brief explanation of how you made your prediction.
            """
            for attempt in range(1, self.max_retries + 1):
                agent = AssistantAgent(
                    name="Player",
                    model_client=self.model_client,
                    system_message="You are a helpful assistant."
                )
                response = await agent.on_messages(
                    [TextMessage(content=instruction, source="user")],
                    cancellation_token=CancellationToken(),
                )
                try:
                    response_data = response.chat_message.content
                    agent_response = AgentResponse.model_validate_json(response_data)
                    move, reasoning = agent_response.prediction, agent_response.reasoning

                    if move in ["Rock", "Paper", "Scissor"]:
                        break
                except (ValidationError, json.JSONDecodeError) as e:
                    print(f"Error parsing response (Attempt {attempt}): {e}")
            else:
                raise ValueError("Model failed to provide a valid response after multiple attempts.")

        outcome = self.determine_winner(move, opponent_move)
        self.update_score(outcome)
        round_result = {
            "Agent Prediction": move,
            "Opponent Move": opponent_move,
            "Reasoning": reasoning,
            "Outcome": outcome
        }
        self.history.append(round_result)
        return round_result

    @staticmethod
    def determine_winner(player_move: str, opponent_move: str) -> int:
        win_conditions = {"Rock": "Scissor", "Scissor": "Paper", "Paper": "Rock"}
        return 1 if player_move == win_conditions.get(opponent_move, "") else 0

    def update_score(self, outcome: int):
        """Updates the score based on the outcome."""
        if outcome == 1:
            self.player_score_game += 1

    def get_history_summary(self) -> str:
        if not self.history:
            return "This is the first round."
        summary = "\n".join(
            [f"Round {i+1}: You guessed {r['Agent Prediction']}, Opponent played {r['Opponent Move']}. Outcome: {r['Outcome']}"
             for i, r in enumerate(self.history)]
        )
        summary += f"\nCurrent Score - You: {self.player_score_game}\n"
        return summary
