import os
import asyncio
from typing import Dict, Literal

from networkx.algorithms.threshold import swap_d
from pydantic import BaseModel
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
import json

from torchgen.dest.ufunc import eligible_for_binary_scalar_specialization

from belief import Belief

from sympy.physics.units import action

# Load API key from environment variable
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
if not OPENAI_API_KEY:
    raise ValueError("Missing OPENAI_API_KEY. Set it as an environment variable.")

# Define the expected response format as a Pydantic model
class AgentResponse(BaseModel):
    action: Literal["A", "B", "X", "Y"]
    reasoning: str

# The ring game simulation class
class Ring:
    debug=False

    def __init__(self, player_id: int, belief: Belief, swap: bool, version: str, model: str, temperature: float, max_retries: int = 3):
        self.player_id = player_id
        self.belief = belief
        self.swap = swap
        self.A, self.B, self.X, self.Y = ("B", "A", "Y", "X") if swap else ("A", "B", "X", "Y")
        self.version = version
        self.model = model
        self.temperature = temperature
        self.max_retries = max_retries  # Maximum retry attempts in case of hallucinations

        is_openai_model = model.startswith("gpt")
        base_url = "https://api.openai.com/v1" if is_openai_model else "http://localhost:11434/v1"

        model_info = {
            "temperature": self.temperature,
            "function_calling": True,
            "parallel_tool_calls": True,
            "family": "unknown",
            "json_output": True,
            "vision": False
        }

        self.model_client = OpenAIChatCompletionClient(
            model=self.model,
            base_url=base_url,
            api_key=OPENAI_API_KEY,
            model_info=model_info,
            response_format=AgentResponse
        )

    async def run(self) -> Dict:
        """Runs the model and ensures a valid response."""
        action_description = (
            ' - `"action"`: Your move ("A" or "B")' if self.player_id == 2
            else ' - `"action"`: Your move ("X" or "Y")'
        )

        XknowingA, XknowingB, YknowingA, YknowingB = (
            (15, 5, 0, 10) if self.version == "a" else
            (8, 7, 7, 8) if self.version == "b" else
            (6, 5, 0, 10) if self.version == "c" else
            (15, 5, 0, 40)
        )

        implicit_belief = f"""
        - If Player 1 chooses {self.X} and Player 2 chooses {self.A}, Player 1 receives {XknowingA} points, and Player 2 receives 10 points.
        - If Player 1 chooses {self.X} and Player 2 chooses {self.B}, Player 1 gets {XknowingB} points, while Player 2 also gets 5 points.
        - If Player 1 chooses {self.Y} and Player 2 chooses {self.A}, Player 1 receives {YknowingA} points, while Player 2 gets 5 points.
        - If Player 1 chooses {self.Y} and Player 2 chooses {self.B}, Player 1 receives {YknowingB} points, and Player 2 gets 0 points.
        """
        explicit_belief = f"""
        {implicit_belief}
        {self.B} is strictly dominated by {self.A}
        """
        given_belief = f"""
        {explicit_belief}
        Player 2 must choose {self.A} if she is rational
        """

        rules = (
            given_belief if self.belief == Belief.GIVEN
            else explicit_belief if self.belief == Belief.EXPLICIT
            else implicit_belief
        )

        instruction = f"""
        We are playing a simple game designed to evaluate your ability to play rationally.
        Your aim is to maximize your own payoff.
        You are Player {self.player_id}.

        ### **Game Rules:**        
        {rules}

        ### **Your Task:**  
        Return your response in JSON format with two keys:  
        {action_description}
        - `"reasoning"`: A brief explanation of how you made your choice.
        """

        if self.debug:
            print(instruction)

        for attempt in range(self.max_retries):
            agent = AssistantAgent(
                name="Player",
                model_client=self.model_client,
                system_message="You are a helpful assistant."
            )

            response = await agent.on_messages(
                [TextMessage(content=instruction, source="user")],
                cancellation_token=CancellationToken(),
            )
            try:
                response_data = response.chat_message.content
                agent_response = AgentResponse.model_validate_json(response_data)  # Parse JSON
                action, reasoning = agent_response.action, agent_response.reasoning
                # Validate values
                if self.player_id == 2 and (action == self.A or action == self.B) or (self.player_id == 1 and (action == self.X or action == self.Y)):
                    rational = 1.0 if self.check_rationality(agent_response) else 0.0
                    return {
                        "action": agent_response.action,
                        "rationality": rational,
                        "reasoning": agent_response.reasoning
                    }
                else:
                    print(f"Invalid response detected (Attempt {attempt+1}): {response_data}")
            except Exception as e:
                print(f"Error parsing response (Attempt {attempt+1}): {e}")
        raise ValueError("Model failed to provide a valid response after multiple attempts.")


    def check_rationality(self, agent_response: AgentResponse) -> bool:
        """Check if the response is rational."""
        if self.player_id == 2:
            return agent_response.action == self.A
        else:
            return agent_response.action == self.X


# Run the async function and return the response
if __name__ == "__main__":
    game_agent = Ring(1, Belief.IMPLICIT, swap = True, version="a", model="llama3", temperature=0.7)
    response_json = asyncio.run(game_agent.run())
    print(response_json)