import os
import asyncio
import csv
import random
from typing import Dict, Literal, List, Callable
from pydantic import BaseModel, ValidationError
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
import json

# Load API key from environment variable
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")
if not OPENAI_API_KEY:
    raise ValueError("Missing OPENAI_API_KEY. Set it as an environment variable.")

CSV_FILE_PATH = "../../data/rps/rps.csv"

# Define the expected response format as a Pydantic model
class AgentResponse(BaseModel):
    move: Literal["Rock", "Paper", "Scissor"]
    motivations: str


# Define RPS simulation class
class RPS:
    def __init__(self, model: str, temperature: float, game_id: int, opponent_strategy_fn: Callable[[List[Dict]], str], max_retries: int = 3):
        self.model = model
        self.temperature = temperature
        self.game_id = game_id
        self.max_retries = max_retries
        self.history: List[Dict] = []
        self.player_score_game = 0
        self.opponent_strategy_fn = opponent_strategy_fn
        is_openai_model = model.startswith("gpt")
        base_url = "https://api.openai.com/v1" if is_openai_model else "http://localhost:11434/v1"
        model_info = {
            "temperature": self.temperature,
            "function_calling": True,
            "parallel_tool_calls": True,
            "family": "unknown",
            "json_output": True,
            "vision": False
        }
        self.model_client = OpenAIChatCompletionClient(
            model=self.model,
            base_url=base_url,
            api_key=OPENAI_API_KEY,
            model_info=model_info,
            response_format=AgentResponse
        )


    async def play_round(self, round_id: int) -> Dict:
        """Plays a single round of RPS."""
        opponent_move = self.opponent_strategy_fn(self.history)
        history_summary = self.get_history_summary()
        instruction = f"""
        We are playing Rock-Paper-Scissors (RPS). The rules are:
        - Rock beats Scissors
        - Scissors beat Paper
        - Paper beats Rock
        - A tie occurs when both players choose the same move.
        Win = 2 points, Tie = 1 point, Loss = 0 points.

        {history_summary}

        Choose your next move: Rock, Paper, or Scissor.
        Return the response in JSON format with `move` and `motivations`.
        """

        for attempt in range(1, self.max_retries + 1):
            agent = AssistantAgent(
                name="Player",
                model_client=self.model_client,
                system_message="You are a helpful assistant."
            )

            response = await agent.on_messages(
                [TextMessage(content=instruction, source="user")],
                cancellation_token=CancellationToken(),
            )

            try:
                response_data = response.chat_message.content
                agent_response = AgentResponse.model_validate_json(response_data)
                move, motivations = agent_response.move, agent_response.motivations

                if move in ["Rock", "Paper", "Scissor"]:
                    outcome = self.determine_winner(move, opponent_move)
                    self.update_score(outcome)  # Use the correct outcome here

                    round_result = {
                        "Agent Move": move,
                        "Opponent Move": opponent_move,
                        "Motivations": motivations,
                        "Outcome": outcome
                    }
                    self.history.append(round_result)

                    return round_result

                print(f"Invalid response detected (Attempt {attempt}): {response_data}")

            except (ValidationError, json.JSONDecodeError) as e:
                print(f"Error parsing response (Attempt {attempt}): {e}")

        raise ValueError("Model failed to provide a valid response after multiple attempts.")

    @staticmethod
    def determine_winner(player_move: str, opponent_move: str) -> int:
        win_conditions = {"Rock": "Scissor", "Scissor": "Paper", "Paper": "Rock"}
        if player_move == opponent_move:
            return 1  # Tie
        elif win_conditions[player_move] == opponent_move:
            return 2  # Win
        else:
            return 0  # Loss

    def update_score(self, outcome: int):
        """Updates the score based on the outcome."""
        if outcome == 2:  # Win
            self.player_score_game += 2
        elif outcome == 1:  # Tie
            self.player_score_game += 1
        # No update needed for Loss (0)

    def get_history_summary(self) -> str:
        if not self.history:
            return "This is the first round."
        summary = "\n".join(
            [f"Round {i+1}: You played {r['Agent Move']}, Opponent played {r['Opponent Move']}. Outcome: {r['Outcome']}"
             for i, r in enumerate(self.history)]
        )
        summary += f"\nCurrent Score - You: {self.player_score_game}\n"
        return summary

