import os
import asyncio
import csv
import random
from rps import RPS
from typing import Dict, Literal, List, Callable
from pydantic import BaseModel, ValidationError
from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.messages import TextMessage
from autogen_core import CancellationToken
from autogen_ext.models.openai import OpenAIChatCompletionClient
import json

CSV_FILE_PATH = "../../data/rps/rps.csv"

# Define RPS Constant Experiment class
class RPSExperiment:
    def __init__(self):
        self.models = ["llama3", "mistral-small", "deepseek-r1"]  # You can also add "gpt-4.5-preview-2025-02-27"
        self.opponent_strategies = {
            "always_rock": lambda history: "Rock",
            "always_paper": lambda history: "Paper",
            "always_scissor": lambda history: "Scissor",
            "R-P": self.loop_R_P,
            "P-S": self.loop_P_S,
            "S-R": self.loop_S_R,
            "R-P-S": self.loop_R_P_S
        }
        self.temperature = 0.7
        self.rounds = 10
        self.num_games_per_config = 10
        self.initialize_csv()


    def loop_R_P(self, history):
        """Alternates between Rock and Paper (R-P)"""
        if len(history) % 2 == 0:
            return "Rock"
        else:
            return "Paper"


    def loop_P_S(self, history):
        """Alternates between Paper and Scissors (P-S)"""
        if len(history) % 2 == 0:
            return "Paper"
        else:
            return "Scissor"


    def loop_S_R(self, history):
        """Alternates between Scissors and Rock (S-R)"""
        if len(history) % 2 == 0:
            return "Scissor"
        else:
            return "Rock"


    def loop_R_P_S(self, history):
        """Alternates between Rock, Paper, and Scissors (R-P-S)"""
        strategies = ["Rock", "Paper", "Scissor"]
        return strategies[len(history) % 3]


    def initialize_csv(self):
        if not os.path.exists(CSV_FILE_PATH):
            os.makedirs(os.path.dirname(CSV_FILE_PATH), exist_ok=True)
            with open(CSV_FILE_PATH, mode="w", newline="") as file:
                writer = csv.writer(file)
                writer.writerow(["idGame", "model", "opponentStrategy", "idRound", "playerMove", "opponentMove", "outcomeRound", "currentPlayerScoreGame", "motivations"])

    def sanitize_motivations(self, motivations: str) -> str:
        sanitized = motivations.replace('"', '""')
        sanitized = sanitized.replace('\n', ' ').replace('\r', '')
        if sanitized and sanitized[0] in ('=', '+', '-', '@'):
            sanitized = "'" + sanitized
        return f'"{sanitized}"'

    def log_to_csv(self, game_id, model, opponent_strategy, round_id, agent_move, opponent_move, outcome, player_score_game, motivations):
        sanitized_motivations = self.sanitize_motivations(motivations)
        with open(CSV_FILE_PATH, mode="a", newline="") as file:
            writer = csv.writer(file)
            writer.writerow([game_id, model, opponent_strategy, round_id, agent_move, opponent_move, outcome, player_score_game, sanitized_motivations])

    async def run_experiment(self):
        """Runs the experiment for all configurations."""
        tasks = []
        game_id = 1
        for model in self.models:
            for strategy_name, strategy_fn in self.opponent_strategies.items():
                for _ in range(self.num_games_per_config):
                    tasks.append(self.run_game(model, strategy_name, strategy_fn, game_id))
                    game_id += 1
        await asyncio.gather(*tasks)


    async def run_game(self, model, opponent_strategy_name, opponent_strategy_fn, game_id):
        game = RPS(model=model, temperature=self.temperature, game_id=game_id, opponent_strategy_fn=opponent_strategy_fn)
        for i in range(1, self.rounds + 1):
            round_data = await game.play_round(i)
            self.log_to_csv(game_id, model, opponent_strategy_name, i, round_data["Agent Move"], round_data["Opponent Move"], round_data["Outcome"], game.player_score_game, round_data["Motivations"])


# Running the experiment:
if __name__ == "__main__":
    experiment = RPSExperiment()
    asyncio.run(experiment.run_experiment())
    print("Experiment completed. Results saved in", CSV_FILE_PATH)